@@ -6,22 +6,17 @@ import warnings
 import numpy as np
 from scipy import stats, sparse
 
-from sklearn.utils.testing import assert_equal
-from sklearn.utils.testing import assert_almost_equal
-from sklearn.utils.testing import assert_raises
-from sklearn.utils.testing import assert_array_equal
-from sklearn.utils.testing import assert_array_almost_equal
-from sklearn.utils.testing import assert_not_in
-from sklearn.utils.testing import assert_less
-from sklearn.utils.testing import assert_warns
-from sklearn.utils.testing import ignore_warnings
-from sklearn.utils.testing import assert_warns_message
-from sklearn.utils.testing import assert_greater
-from sklearn.utils.testing import assert_greater_equal
+import pytest
+
+from sklearn.utils._testing import assert_almost_equal
+from sklearn.utils._testing import assert_array_equal
+from sklearn.utils._testing import assert_array_almost_equal
+from sklearn.utils._testing import assert_warns
+from sklearn.utils._testing import ignore_warnings
+from sklearn.utils._testing import assert_warns_message
 from sklearn.utils import safe_mask
 
-from sklearn.datasets.samples_generator import (make_classification,
-                                                make_regression)
+from sklearn.datasets import make_classification, make_regression
 from sklearn.feature_selection import (
     chi2, f_classif, f_oneway, f_regression, mutual_info_classif,
     mutual_info_regression, SelectPercentile, SelectKBest, SelectFpr,
@@ -193,10 +188,10 @@ def test_select_percentile_classif_sparse():
     X_r2inv = univariate_filter.inverse_transform(X_r2)
     assert sparse.issparse(X_r2inv)
     support_mask = safe_mask(X_r2inv, support)
-    assert_equal(X_r2inv.shape, X.shape)
+    assert X_r2inv.shape == X.shape
     assert_array_equal(X_r2inv[:, support_mask].toarray(), X_r.toarray())
     # Check other columns are empty
-    assert_equal(X_r2inv.getnnz(), X_r.getnnz())
+    assert X_r2inv.getnnz() == X_r.getnnz()
 
 
 ##############################################################################
@@ -245,7 +240,7 @@ def test_select_kbest_zero():
     assert_array_equal(support, gtruth)
     X_selected = assert_warns_message(UserWarning, 'No features were selected',
                                       univariate_filter.transform, X)
-    assert_equal(X_selected.shape, (20, 0))
+    assert X_selected.shape == (20, 0)
 
 
 def test_select_heuristics_classif():
@@ -327,12 +322,14 @@ def test_invalid_percentile():
     X, y = make_regression(n_samples=10, n_features=20,
                            n_informative=2, shuffle=False, random_state=0)
 
-    assert_raises(ValueError, SelectPercentile(percentile=-1).fit, X, y)
-    assert_raises(ValueError, SelectPercentile(percentile=101).fit, X, y)
-    assert_raises(ValueError, GenericUnivariateSelect(mode='percentile',
-                                                      param=-1).fit, X, y)
-    assert_raises(ValueError, GenericUnivariateSelect(mode='percentile',
-                                                      param=101).fit, X, y)
+    with pytest.raises(ValueError):
+        SelectPercentile(percentile=-1).fit(X, y)
+    with pytest.raises(ValueError):
+        SelectPercentile(percentile=101).fit(X, y)
+    with pytest.raises(ValueError):
+        GenericUnivariateSelect(mode='percentile', param=-1).fit(X, y)
+    with pytest.raises(ValueError):
+        GenericUnivariateSelect(mode='percentile', param=101).fit(X, y)
 
 
 def test_select_kbest_regression():
@@ -371,7 +368,7 @@ def test_select_heuristics_regression():
         assert_array_equal(X_r, X_r2)
         support = univariate_filter.get_support()
         assert_array_equal(support[:5], np.ones((5, ), dtype=np.bool))
-        assert_less(np.sum(support[5:] == 1), 3)
+        assert np.sum(support[5:] == 1) < 3
 
 
 def test_boundary_case_ch2():
@@ -408,7 +405,9 @@ def test_boundary_case_ch2():
     assert_array_equal(support_fwe, np.array([True, False]))
 
 
-def test_select_fdr_regression():
+@pytest.mark.parametrize("alpha", [0.001, 0.01, 0.1])
+@pytest.mark.parametrize("n_informative", [1, 5, 10])
+def test_select_fdr_regression(alpha, n_informative):
     # Test that fdr heuristic actually has low FDR.
     def single_fdr(alpha, n_informative, random_state):
         X, y = make_regression(n_samples=150, n_features=20,
@@ -434,20 +433,18 @@ def test_select_fdr_regression():
                                 (num_true_positives + num_false_positives))
         return false_discovery_rate
 
-    for alpha in [0.001, 0.01, 0.1]:
-        for n_informative in [1, 5, 10]:
-            # As per Benjamini-Hochberg, the expected false discovery rate
-            # should be lower than alpha:
-            # FDR = E(FP / (TP + FP)) <= alpha
-            false_discovery_rate = np.mean([single_fdr(alpha, n_informative,
-                                                       random_state) for
-                                            random_state in range(100)])
-            assert_greater_equal(alpha, false_discovery_rate)
+    # As per Benjamini-Hochberg, the expected false discovery rate
+    # should be lower than alpha:
+    # FDR = E(FP / (TP + FP)) <= alpha
+    false_discovery_rate = np.mean([single_fdr(alpha, n_informative,
+                                               random_state) for
+                                    random_state in range(100)])
+    assert alpha >= false_discovery_rate
 
-            # Make sure that the empirical false discovery rate increases
-            # with alpha:
-            if false_discovery_rate != 0:
-                assert_greater(false_discovery_rate, alpha / 10)
+    # Make sure that the empirical false discovery rate increases
+    # with alpha:
+    if false_discovery_rate != 0:
+        assert false_discovery_rate > alpha / 10
 
 
 def test_select_fwe_regression():
@@ -466,7 +463,7 @@ def test_select_fwe_regression():
     gtruth = np.zeros(20)
     gtruth[:5] = 1
     assert_array_equal(support[:5], np.ones((5, ), dtype=np.bool))
-    assert_less(np.sum(support[5:] == 1), 2)
+    assert np.sum(support[5:] == 1) < 2
 
 
 def test_selectkbest_tiebreaking():
@@ -478,12 +475,12 @@ def test_selectkbest_tiebreaking():
     for X in Xs:
         sel = SelectKBest(dummy_score, k=1)
         X1 = ignore_warnings(sel.fit_transform)([X], y)
-        assert_equal(X1.shape[1], 1)
+        assert X1.shape[1] == 1
         assert_best_scores_kept(sel)
 
         sel = SelectKBest(dummy_score, k=2)
         X2 = ignore_warnings(sel.fit_transform)([X], y)
-        assert_equal(X2.shape[1], 2)
+        assert X2.shape[1] == 2
         assert_best_scores_kept(sel)
 
 
@@ -495,12 +492,12 @@ def test_selectpercentile_tiebreaking():
     for X in Xs:
         sel = SelectPercentile(dummy_score, percentile=34)
         X1 = ignore_warnings(sel.fit_transform)([X], y)
-        assert_equal(X1.shape[1], 1)
+        assert X1.shape[1] == 1
         assert_best_scores_kept(sel)
 
         sel = SelectPercentile(dummy_score, percentile=67)
         X2 = ignore_warnings(sel.fit_transform)([X], y)
-        assert_equal(X2.shape[1], 2)
+        assert X2.shape[1] == 2
         assert_best_scores_kept(sel)
 
 
@@ -514,12 +511,12 @@ def test_tied_pvalues():
     for perm in itertools.permutations((0, 1, 2)):
         X = X0[:, perm]
         Xt = SelectKBest(chi2, k=2).fit_transform(X, y)
-        assert_equal(Xt.shape, (2, 2))
-        assert_not_in(9998, Xt)
+        assert Xt.shape == (2, 2)
+        assert 9998 not in Xt
 
         Xt = SelectPercentile(chi2, percentile=67).fit_transform(X, y)
-        assert_equal(Xt.shape, (2, 2))
-        assert_not_in(9998, Xt)
+        assert Xt.shape == (2, 2)
+        assert 9998 not in Xt
 
 
 def test_scorefunc_multilabel():
@@ -529,12 +526,12 @@ def test_scorefunc_multilabel():
     y = [[1, 1], [0, 1], [1, 0]]
 
     Xt = SelectKBest(chi2, k=2).fit_transform(X, y)
-    assert_equal(Xt.shape, (3, 2))
-    assert_not_in(0, Xt)
+    assert Xt.shape == (3, 2)
+    assert 0 not in Xt
 
     Xt = SelectPercentile(chi2, percentile=67).fit_transform(X, y)
-    assert_equal(Xt.shape, (3, 2))
-    assert_not_in(0, Xt)
+    assert Xt.shape == (3, 2)
+    assert 0 not in Xt
 
 
 def test_tied_scores():
@@ -555,7 +552,7 @@ def test_nans():
     X = [[0, 1, 0], [0, -1, -1], [0, .5, .5]]
     y = [1, 0, 1]
 
-    for select in (SelectKBest(f_classif, 2),
+    for select in (SelectKBest(f_classif, k=2),
                    SelectPercentile(f_classif, percentile=67)):
         ignore_warnings(select.fit)(X, y)
         assert_array_equal(select.get_support(indices=True), np.array([1, 2]))
@@ -567,19 +564,22 @@ def test_score_func_error():
 
     for SelectFeatures in [SelectKBest, SelectPercentile, SelectFwe,
                            SelectFdr, SelectFpr, GenericUnivariateSelect]:
-        assert_raises(TypeError, SelectFeatures(score_func=10).fit, X, y)
+        with pytest.raises(TypeError):
+            SelectFeatures(score_func=10).fit(X, y)
 
 
 def test_invalid_k():
     X = [[0, 1, 0], [0, -1, -1], [0, .5, .5]]
     y = [1, 0, 1]
 
-    assert_raises(ValueError, SelectKBest(k=-1).fit, X, y)
-    assert_raises(ValueError, SelectKBest(k=4).fit, X, y)
-    assert_raises(ValueError,
-                  GenericUnivariateSelect(mode='k_best', param=-1).fit, X, y)
-    assert_raises(ValueError,
-                  GenericUnivariateSelect(mode='k_best', param=4).fit, X, y)
+    with pytest.raises(ValueError):
+        SelectKBest(k=-1).fit(X, y)
+    with pytest.raises(ValueError):
+        SelectKBest(k=4).fit(X, y)
+    with pytest.raises(ValueError):
+        GenericUnivariateSelect(mode='k_best', param=-1).fit(X, y)
+    with pytest.raises(ValueError):
+        GenericUnivariateSelect(mode='k_best', param=4).fit(X, y)
 
 
 def test_f_classif_constant_feature():
@@ -608,7 +608,7 @@ def test_no_feature_selected():
         assert_array_equal(selector.get_support(), np.zeros(10))
         X_selected = assert_warns_message(
             UserWarning, 'No features were selected', selector.transform, X)
-        assert_equal(X_selected.shape, (40, 0))
+        assert X_selected.shape == (40, 0)
 
 
 def test_mutual_info_classif():
