@@ -4,9 +4,6 @@ Testing for Theil-Sen module (sklearn.linear_model.theil_sen)
 
 # Author: Florian Wilhelm <florian.wilhelm@gmail.com>
 # License: BSD 3 clause
-
-from __future__ import division, print_function, absolute_import
-
 import os
 import sys
 from contextlib import contextmanager
@@ -17,11 +14,9 @@ from scipy.linalg import norm
 from scipy.optimize import fmin_bfgs
 from sklearn.exceptions import ConvergenceWarning
 from sklearn.linear_model import LinearRegression, TheilSenRegressor
-from sklearn.linear_model.theil_sen import _spatial_median, _breakdown_point
-from sklearn.linear_model.theil_sen import _modified_weiszfeld_step
-from sklearn.utils.testing import (
-        assert_almost_equal, assert_greater, assert_less, raises,
-)
+from sklearn.linear_model._theil_sen import _spatial_median, _breakdown_point
+from sklearn.linear_model._theil_sen import _modified_weiszfeld_step
+from sklearn.utils._testing import assert_almost_equal, assert_raises
 
 
 @contextmanager
@@ -166,7 +161,7 @@ def test_theil_sen_1d():
     X, y, w, c = gen_toy_problem_1d()
     # Check that Least Squares fails
     lstq = LinearRegression().fit(X, y)
-    assert_greater(np.abs(lstq.coef_ - w), 0.9)
+    assert np.abs(lstq.coef_ - w) > 0.9
     # Check that Theil-Sen works
     theil_sen = TheilSenRegressor(random_state=0).fit(X, y)
     assert_array_almost_equal(theil_sen.coef_, w, 1)
@@ -177,7 +172,7 @@ def test_theil_sen_1d_no_intercept():
     X, y, w, c = gen_toy_problem_1d(intercept=False)
     # Check that Least Squares fails
     lstq = LinearRegression(fit_intercept=False).fit(X, y)
-    assert_greater(np.abs(lstq.coef_ - w - c), 0.5)
+    assert np.abs(lstq.coef_ - w - c) > 0.5
     # Check that Theil-Sen works
     theil_sen = TheilSenRegressor(fit_intercept=False,
                                   random_state=0).fit(X, y)
@@ -189,7 +184,7 @@ def test_theil_sen_2d():
     X, y, w, c = gen_toy_problem_2d()
     # Check that Least Squares fails
     lstq = LinearRegression().fit(X, y)
-    assert_greater(norm(lstq.coef_ - w), 1.0)
+    assert norm(lstq.coef_ - w) > 1.0
     # Check that Theil-Sen works
     theil_sen = TheilSenRegressor(max_subpopulation=1e3,
                                   random_state=0).fit(X, y)
@@ -199,34 +194,34 @@ def test_theil_sen_2d():
 
 def test_calc_breakdown_point():
     bp = _breakdown_point(1e10, 2)
-    assert_less(np.abs(bp - 1 + 1 / (np.sqrt(2))), 1.e-6)
+    assert np.abs(bp - 1 + 1 / (np.sqrt(2))) < 1.e-6
 
 
-@raises(ValueError)
 def test_checksubparams_negative_subpopulation():
     X, y, w, c = gen_toy_problem_1d()
-    TheilSenRegressor(max_subpopulation=-1, random_state=0).fit(X, y)
+    theil_sen = TheilSenRegressor(max_subpopulation=-1, random_state=0)
+    assert_raises(ValueError, theil_sen.fit, X, y)
 
 
-@raises(ValueError)
 def test_checksubparams_too_few_subsamples():
     X, y, w, c = gen_toy_problem_1d()
-    TheilSenRegressor(n_subsamples=1, random_state=0).fit(X, y)
+    theil_sen = TheilSenRegressor(n_subsamples=1, random_state=0)
+    assert_raises(ValueError, theil_sen.fit, X, y)
 
 
-@raises(ValueError)
 def test_checksubparams_too_many_subsamples():
     X, y, w, c = gen_toy_problem_1d()
-    TheilSenRegressor(n_subsamples=101, random_state=0).fit(X, y)
+    theil_sen = TheilSenRegressor(n_subsamples=101, random_state=0)
+    assert_raises(ValueError, theil_sen.fit, X, y)
 
 
-@raises(ValueError)
 def test_checksubparams_n_subsamples_if_less_samples_than_features():
     random_state = np.random.RandomState(0)
     n_samples, n_features = 10, 20
     X = random_state.normal(size=(n_samples, n_features))
     y = random_state.normal(size=n_samples)
-    TheilSenRegressor(n_subsamples=9, random_state=0).fit(X, y)
+    theil_sen = TheilSenRegressor(n_subsamples=9, random_state=0)
+    assert_raises(ValueError, theil_sen.fit, X, y)
 
 
 def test_subpopulation():
@@ -260,9 +255,9 @@ def test_theil_sen_parallel():
     X, y, w, c = gen_toy_problem_2d()
     # Check that Least Squares fails
     lstq = LinearRegression().fit(X, y)
-    assert_greater(norm(lstq.coef_ - w), 1.0)
+    assert norm(lstq.coef_ - w) > 1.0
     # Check that Theil-Sen works
-    theil_sen = TheilSenRegressor(n_jobs=-1,
+    theil_sen = TheilSenRegressor(n_jobs=2,
                                   random_state=0,
                                   max_subpopulation=2e3).fit(X, y)
     assert_array_almost_equal(theil_sen.coef_, w, 1)
